mod database;
mod mcdu;
mod sheet;
mod sheet_listing;

use std::{path::PathBuf, process};

use clap::Parser;
use database::Database;
use env_logger::Env;
use gtk::prelude::*;
use log::error;
use mcdu::McduModel;
use relm4::prelude::*;
use sheet_listing::{SheetListingInput, SheetListingModel};

struct AppModel {
    mcdu: Controller<McduModel>,
    sheet_listing: Controller<SheetListingModel>,
}

#[derive(Debug)]
enum AppInput {
    SearchStarted(String),
    SheetPressed(PathBuf),
}

#[relm4::component]
impl SimpleComponent for AppModel {
    type Input = AppInput;
    type Output = ();
    type Init = PathBuf;

    view! {
        #[root]
        gtk::Window{
            set_default_width: 300,
            set_default_height: 300,
            set_title: Some("Play music!!!"),
            set_maximized: true,
            gtk::Box {
                set_orientation: gtk::Orientation::Horizontal,
                gtk::Box {
                    set_orientation: gtk::Orientation::Vertical,
                    set_hexpand: true,
                    gtk::ScrolledWindow {
                        model.sheet_listing.widget(),
                        set_vexpand: true,
                        set_hexpand: true,
                        // set_hscrollbar_policy: PolicyType::Never,
                    },
                },
                model.mcdu.widget(),
            }
        }
    }

    fn init(
        path: Self::Init,
        window: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> relm4::ComponentParts<Self> {
        relm4_icons::initialize_icons();

        let mcdu = McduModel::builder()
            .launch(())
            .forward(sender.input_sender(), |response| match response {
                mcdu::McduOutput::SearchStarted(query) => AppInput::SearchStarted(query),
            });

        let sheet_listing =
            SheetListingModel::builder()
                .launch(path)
                .forward(sender.input_sender(), |response| match response {
                    sheet_listing::SheetPressedMessage::SheetPressed(path) => {
                        AppInput::SheetPressed(path)
                    }
                });

        let model = AppModel {
            mcdu,
            sheet_listing,
        };

        let widgets = view_output!();

        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, _sender: ComponentSender<Self>) {
        match message {
            AppInput::SheetPressed(sheet) => opener::open(sheet).unwrap(),
            AppInput::SearchStarted(query) => self.sheet_listing.emit(SheetListingInput { query }),
        }
    }
}

#[derive(Parser)]
#[command(author, version, about)]
struct Cli {
    directory: PathBuf,
}

#[tokio::main]
async fn main() {
    // dotenvy::dotenv().unwrap();
    env_logger::Builder::from_env(Env::default().default_filter_or("debug")).init();
    let cli = Cli::parse();
    if !cli.directory.is_dir() {
        error!("Sheet folder path is no dir or does not exist");
        process::exit(1);
    }

    let database = Database::setup("./testdb.db").await.unwrap();
    // database.insert_sheet(Sheet::new_debug()).await.unwrap();
    let sheets = database.fetch_all_sheets().await.unwrap();

    let app = RelmApp::new("de.frajul.sheet-organizer");
    // Pass empty command line args to allow my own parsing
    app.with_args(Vec::new()).run::<AppModel>(cli.directory);
}
